/* Eingaben ber ein Hyperterminal werden als ECHO zurckgesendet oder auf dem LCD ausgegeben
 *
 * 03/2005 Florian Schaeffer
 */

#include "uart.h"
#include <avr/signal.h>
#include <avr/interrupt.h>

#define UART_BAUD_CALC(UART_BAUD_RATE,F_CPU) ((F_CPU)/((UART_BAUD_RATE)*16l)-1)

#define RBUFFLEN 40 //Pufferlnge fr seriellen Empfang

volatile unsigned char rbuff[RBUFFLEN];		// Ringpuffer
volatile uint8_t 	rbuffpos, 					// Position, die als nchstes gelesen werden mu im Ringpuffer
					rbuffcnt, 					// Anzahl zu lesender Zeichen im Puffer
					udr_data;					// Daten aus dem UART (volatile, damit nicht wegoptimiert wird vom Prprozessor)

// Interruptroutine, die Zeichen aus dem UART sofort ausliest, wenn empfangen
SIGNAL (SIG_UART_RECV) 		
{
	udr_data= UDR; 	//Byte auf jeden Fall abholen, sonst Endlosinterrupt
	
	if(rbuffcnt < RBUFFLEN)		// keinen Zeichen in einem vollen Ringpuffer berschreiben
		rbuff[(rbuffpos+rbuffcnt++) % RBUFFLEN] = udr_data;	// welche Position? Gelesene Zeichenpos + Anzahl Zeichen MODULO Pufferlnge 
																// (von 0 wieder anfangen, wenn Ende erreicht)
}

// Nchstes zu lesendes Zeichen aus Ringpuffer zurckgeben
unsigned char ser_getc (void)		
{
	unsigned char c;

	while(!rbuffcnt);   		// Warte bis ein Zeichen vorhanden ist
	
	cli();						// Interruptbehandlung kurz aussetzen. Ab jetzt mu es schnell gehen (wenig Befehle), damit Zeichen, die 
								// ab jetzt eintreffen nicht verloren gehen.
	rbuffcnt--;					// anschl. ein Zeichen weniger zum ausgeben
	c = rbuff [rbuffpos++];	// Zeichen holen, was nach dem bereits gelesenen liegt
	if (rbuffpos >= RBUFFLEN)  rbuffpos = 0;	// wenn hinterstes Zeichen (rechts im Puffer) gelesen wurde, dann wieder vorne anfangen

	sei();						// Interruptbehandlung wieder aktivieren

	return (c);				// Zeichen zurckgeben
}

// Ein Zeichen senden
void uart_putc(unsigned char c)
{
    while(!(UCSRA & (1 << UDRE)));   	// warte, bis UDR bereit 

    UDR = c;    						// sende Zeichen 
}

// Einen String senden
void uart_puts (char *s)
{
    while (*s)
    {   // so lange *s != NULL 
        uart_putc(*s);
        s++;
    }
}

// USART initialisieren
void uart_ini ()
{
	sei();	// Interruptbehandlung aktivieren

	UCSRB |= (1 << TXEN);											// UART TX (senden) einschalten
	UCSRB |= (1 << RXEN );											// UART RX (empfangen) einschalten
	UCSRB |= (1 << RXCIE);											// Interruptauslsung fr eingehende Daten aktivieren
	UCSRC |= (1<<URSEL)|(3<<UCSZ0);									// Asynchron, 8N1
	UBRRH=(uint8_t)(UART_BAUD_CALC(UART_BAUD_RATE,F_CPU)>>8);		// Baudrate whlen
	UBRRL=(uint8_t)UART_BAUD_CALC(UART_BAUD_RATE,F_CPU);
}
